# -*- coding: utf-8 -*-

from jamip.db.connect import Structure,Read,Write
from jamip.db.materials.molStructure import MolStructure
from jamip.db.materials.molComposition import MolComposition
from jamip.db.modeling.structureFactory import StructureFactory
import numpy as np
import os

# create Output
output=['../Output/forHTCalculations/single', 
        '../Output/forHTCalculations/double',
        '../Output/2D/non-passivation',
        '../Output/2D/passivation',
        '../Output/defect/moire_supercell']
for path in output: os.system('mkdir -p {}'.format(path))

# CsPbBr3
raw=Read('../Input/CsPbBr3.vasp', dtype='poscar').run()
s0=Structure().create(raw_structure=raw)


# -------------------- for HT calculations (ABX3) --------------------
A=['Na', 'K', 'Rb', 'Cs']
B=['Ge', 'Sn', 'Pb']
X=['Cl', 'Br', 'Br']

for a0 in A:
    for b0 in B:
        for x0 in X:
            sf=StructureFactory(structure=s0)
            sf.substitute_atoms(atoms=s0.get_atoms_of_element('Cs'),
                                symbol_of_elements=a0)
            sf.substitute_atoms(atoms=s0.get_atoms_of_element('Pb'), 
                                symbol_of_elements=b0)                
            sf.substitute_atoms(atoms=s0.get_atoms_of_element('Br'), 
                                symbol_of_elements=x0)  
            Write(structure=sf.structure, 
                  path='../Output/forHTCalculations/single/{}{}{}3.vasp'.format(a0, b0, x0), 
                  dtype='poscar').run()

# molecule
raw=Read('../Input/molecule/CH3NH3.xyz', dtype='xyz').run()
MA=MolStructure().create(raw_structure=raw)
MA.label='MA'

raw=Read('../Input/molecule/CH3CH2NH3.mol', dtype='mol').run()
EA=MolStructure().create(raw_structure=raw)
EA.label='EA'

raw=Read('../Input/molecule/CH3NH2CH3.mol', dtype='mol').run()
DEA=MolStructure().create(raw_structure=raw)
DEA.label='DEA'

raw=Read('../Input/molecule/CH3ONH2.mol', dtype='mol').run()
FM=MolStructure().create(raw_structure=raw)
FM.label='FM'

mol_A=[MA, EA, DEA, FM]
for ma0 in mol_A:
    sf=StructureFactory(structure=s0)
    sf.del_atoms(atoms=s0.get_atoms_of_element('Cs'))
    sf.insertMolecule(structure_of_molecule=ma0, 
                      position_in_molecule=ma0.mass_center,  
                      position=s0.get_atoms_of_element('Cs')[0].position)
    Write(structure=sf.structure, 
          path='../Output/forHTCalculations/single/{}PbBr3.vasp'.format(ma0.label), 
          dtype='poscar').run()
    
    
# save to DB
raw=Read('../Input/molecule/CN2H5.mol', dtype='mol').run()
FA=MolStructure().create(raw_structure=raw)
FA.label='FA'
FA.update(isPersist=True)

raw=Read('../Input/molecule/CN3H6.mol', dtype='mol').run()
GA=MolStructure().create(raw_structure=raw)
GA.label='GA'
GA.update(isPersist=True)

raw=Read('../Input/molecule/NH2NH3.mol', dtype='mol').run()
DA=MolStructure().create(raw_structure=raw)
DA.label='DA'
DA.update(isPersist=True)

raw=Read('../Input/molecule/NH3OH.mol', dtype='mol').run()
HA=MolStructure().create(raw_structure=raw)
HA.label='HA'
HA.update(isPersist=True)

raw=Read('../Input/molecule/NH4.mol', dtype='mol').run()
M=MolStructure().create(raw_structure=raw)
M.label='M'
M.update(isPersist=True)

# read molecule from JAMIP's DB
mol_A_in_DB=[MolComposition.objects.get(formula='CN2H5').structure_set.all()[0], 
             MolComposition.objects.get(formula='CH6N3').structure_set.all()[0],
             MolComposition.objects.get(formula='N2H5').structure_set.all()[0],
             MolComposition.objects.get(formula='NOH4').structure_set.all()[0],
             MolComposition.objects.get(formula='H4N').structure_set.all()[0]]

for ma0 in mol_A_in_DB:
    sf=StructureFactory(structure=s0)
    sf.del_atoms(atoms=s0.get_atoms_of_element('Cs'))
    sf.insertMolecule(structure_of_molecule=ma0,
                      position_in_molecule=ma0.mass_center,
                      position=s0.get_atoms_of_element('Cs')[0].position) 
    Write(structure=sf.structure, 
          path='../Output/forHTCalculations/single/{}PbBr3.vasp'.format(ma0.label), 
          dtype='poscar').run()


# -------------------- double perovisktes --------------------
B1=['Na', 'Ag']
B2=['In', 'Bi']
for b1 in B1:
    for b2 in B2:
        sf=StructureFactory(structure=s0)
        sf.supercell(dim=[2,2,2])
        sf.substitute_atoms(atoms=[['Pb', 0.25, 0.25, 0.25],
                                  ['Pb', 0.75, 0.25, 0.75],
                                  ['Pb', 0.75, 0.75, 0.25], 
                                  ['Pb', 0.25, 0.75, 0.75]], 
                            symbol_of_elements='Ag')
        sf.substitute_atoms(atoms=[['Pb', 0.75, 0.25, 0.25],
                                  ['Pb', 0.25, 0.25, 0.75],
                                  ['Pb', 0.25, 0.75, 0.25],
                                  ['Pb', 0.75, 0.75, 0.75]],
                            symbol_of_elements='Bi').primitive()
        Write(structure=sf.structure, 
              path='../Output/forHTCalculations/double/Cs{}{}Br3.vasp'.format(b1, 
                                                                              b2), 
              dtype='poscar').run()


# -------------------- 2D materials --------------------
sf=StructureFactory(structure=s0)
sf.supercell(dim=[1, 1, 3])
sf.removeUnit(unit=[0, 0.3, 2], 
              isMoveAtoms=False).removeUnit(unit=[0.7, 1, 2], 
                                            isMoveAtoms=False)

dz=2 # z-direction
for i in range(0, 5):
    sf2=StructureFactory(structure=sf.structure)
    sf2.addUnit(unit=[0.33333, 0.66667, dz], 
                nrepeat=i)    
    Write(structure=sf2.structure, 
          path='../Output/2D/non-passivation/CsPbBr3-2d-{}.vasp'.format(i+1), 
          dtype='poscar').run()
    
# -------------------- surface passivation --------------------
dz=2 # z-direction
for i in range(0, 5):
    sf2=StructureFactory(structure=sf.structure)
    sf2.addUnit(unit=[0.33333, 0.66667, dz], nrepeat=i)    
    
    Cs=sorted(sf2.structure.get_atoms_of_element('Cs'), key=lambda atom: atom.position[dz]) # sorte Cs along z direction

    mol_A=[MA, EA, DEA, FM]
    for ma0 in mol_A:
        sf3=StructureFactory(structure=sf2.structure)
        # left
        sf3.del_atoms(atoms=[Cs[0]])
        sf3.insertMolecule(structure_of_molecule=ma0, 
                           position_in_molecule=ma0.mass_center,
                           position=Cs[0].position)
        # right
        sf3.del_atoms(atoms=[Cs[-1]])
        sf3.insertMolecule(structure_of_molecule=ma0, 
                           position_in_molecule=ma0.mass_center,
                           position=Cs[-1].position)
        label = ma0.label
        Write(structure=sf3.structure, 
              path='../Output/2D/passivation/CsPbBr3-2d-{}-{}.vasp'.format(i+1, 
                                                                           label), 
              dtype='poscar').run()
    

# -------------------- defect --------------------
# -------------------- random vacancy --------------------
from random import randint

sf=StructureFactory(structure=s0)
sf.supercell(dim=[3, 3, 3])

Br=sf.structure.get_atoms_of_element('Br')

indexs=[randint(0, len(Br)) for i in range(0, 3)]
del_Br=[Br[i] for i in indexs]

sf.del_atoms(atoms=del_Br)

Write(structure=sf.structure, 
      path='../Output/defect/CsPbBr3_vacancy.vasp', 
      dtype='poscar').run()

# -------------------- twin --------------------
sf=StructureFactory(structure=s0)
sf.supercell(dim=[1, 8, 8])

degree=5
dzs=np.linspace(0, 1, 50)
for i in range(0, len(dzs)-1):
    sf.rotation(atoms=sf.getUnit(unit=[dzs[i], dzs[i+1], 2]), 
                axis=[1, 0, 0], 
                theta=[degree, 'd'],
                origin=[0, 0, dzs[i]])


sf.removeUnit(unit=[0, 0.25, 2], 
              isMoveAtoms=False).removeUnit(unit=[0.75, 1, 2], 
                                            isMoveAtoms=False)
sf.removeUnit(unit=[0.51, 1, 1], 
             isMoveAtoms=False)                                            
sf.vacuum(direction=[0, 0, -0.5, 'Direct'], isCenter=False)

add_atoms=[]
for atom0 in sf.structure.atoms:
    if 0.05 < atom0.position[1] < 0.45:
        fa0=atom0.to_formated_atom()
        fa0[2] = 1-fa0[2] # mirror image with the y direction
        add_atoms.append(fa0)
sf.add_atoms(atoms=add_atoms)

Write(structure=sf.structure, 
      path='../Output/defect/CsPbBr3_twist.vasp', 
      dtype='poscar').run() 


# -------------------- spiral strain --------------------
sf=StructureFactory(structure=s0)
sf.supercell(dim=[3, 3, 10]).vacuum(direction=[0, 0, 10, 'Cartesian'], 
                                    isCenter=True).removeUnit(unit=[0.88, 1, 2], 
                                                              isCenter=True)

for i in range(0, 2):
    sf.removeUnit(unit=[0, 0.3, i], 
                  isMoveAtoms=False).removeUnit(unit=[0.7, 1, i], 
                                                isMoveAtoms=False)

degree=0.5
dzs=np.linspace(0, 1, 50)
for i in range(0, len(dzs)-1):
    sf.rotation(atoms=sf.getUnit(unit=[dzs[i], dzs[i+1], 2], 
                                 symbol_of_atoms=['Br', 'Cs']), 
                axis=[0, 0, 1], 
                theta=[degree*i, 'd'],
                origin=[0.5, 0.5, 0])

Write(structure=sf.structure, 
      path='../Output/defect/CsPbBr3_spiral_strain.vasp', 
      dtype='poscar').run()


# -------------------- moire supercell --------------------
# MoS2
raw=Read('../Input/MoS2.vasp', dtype='poscar').run()
s0=Structure().create(raw_structure=raw)

sf=StructureFactory(structure=s0)

sf.removeUnit(unit=[0, 0.1, 2], 
              isMoveAtoms=False).removeUnit(unit=[0.9, 1, 2], 
                                            isMoveAtoms=False)
sf.supercell(dim=[10, 10, 1])

degree=1
sf.rotation(atoms=sf.getUnit(unit=[0.5, 1, 2]), 
            axis=[0, 0, 1, ], 
            theta=[degree, 'd'],
            origin=[0, 0, 0])

Write(structure=sf.structure, 
      path='../Output/defect/moire_supercell/MoS2_moire.vasp', 
      dtype='poscar').run()




